import random
import time

def run(bridge, duration=15, fade_up_delay=0, fade_down_delay=0.01, on_duration=0.5, off_duration=0,
        colors=['yellow'], color_changes=True):
    '''An effect that simulates yellow factory warning flashers.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    duration -- length of time the effect will last (default 15)
    fade_up_delay -- delay between fade up commands (default 0)
    fade_down_delay -- delay between fade down commands (default 0.01)
    on_duration -- length of time for the lights to remain in the on position (default 0.5)
    off_duration -- length of time for the lights to remain in the off position (default 0)
    colors -- list of possible colors to choose from (default: ['yellow']])
              `None`: Random selection of colors
    color_changes -- boolean that judges if colors will be changed inside the effect (default True)
    '''

    # Keep track of time in effect
    start_time = time.time()

    if color_changes:
        # Set white to be easily accessible via mode_down()
        if 'white' in colors:
            bridge.white()

    # Main loop
    while True:
        # Flash the lights off, then on
        bridge.all_off()
        time.sleep(off_duration)
        bridge.all_on()

        if color_changes:
            if colors:
                # If colors are provided, choose a specified color
                current_color = colors[int(random.random() * len(colors))]
                if current_color == 'white':
                    bridge.mode_down()
                else:
                    bridge.set_color(current_color)
            else:
                # If colors are NOT provided, choose a random color
                bridge.set_color_hex(chr(int(random.random() * 255)))

        
        # Swell lights to max brightness
        if hasattr(bridge, 'set_brightness') and callable(getattr(bridge, 'set_brightness')):
            for i in range(2,28):
                bridge.set_brightness(i)
                time.sleep(fade_up_delay)
        else:
            for i in range(0, 9):
                bridge.brightness_up()
                time.sleep(fade_up_delay)

        # Check if effect duration has past
        if time.time() - start_time > duration:
            break

        time.sleep(on_duration)

        # Swell lights to min brightness
        if hasattr(bridge, 'set_brightness') and callable(getattr(bridge, 'set_brightness')):
            for i in range(2,28):
                bridge.set_brightness(29-i)
                time.sleep(fade_down_delay)
        else:
            for i in range(0, 9):
                bridge.brightness_down()
                time.sleep(fade_down_delay)



import os
for module in os.listdir(os.path.dirname(__file__)):
    if module == '__init__.py':
        continue
    if module[-3:] != '.py':
        if os.path.isdir(os.path.join(os.path.dirname(__file__), module)):
            __import__(module, locals(), globals())
        continue
    __import__(module[:-3], locals(), globals())
del module



import random
import time

def run(bridge, duration=15, on_duration=0.3, off_duration=0.01,
        colors=None):
    '''A colorful strobe light effect.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    duration -- length of time the effect will last (default 15)
    on_duration -- length of time the lights remain in the on position (default 0.3)
    off_duration -- length of time the lights remain in the off position (default 0.01)
    colors -- list of possible colors to choose from (default: None)
              `None`: Random selection of colors
    '''
    # Keep track of time in effect
    start_time = time.time()

    if colors:
        # Set white to be easily accessible via mode_down()
        if 'white' in colors:
            bridge.white()

    # Main loop
    while True:
        # Flash the lights off, then on
        if off_duration:
            bridge.all_off()
            time.sleep(off_duration)
            bridge.all_on()

        if colors:
            # If colors are provided, choose a specified color
            current_color = colors[int(random.random() * len(colors))]
            if current_color == 'white':
                bridge.mode_down()
            else:
                bridge.set_color(current_color)
        else:
            # If colors are NOT provided, choose a random color
            bridge.set_color_hex(chr(int(random.random() * 255)))

        # Check if effect duration has past
        if time.time() - start_time > duration:
            break

        # Leave the bridge on the same color
        time.sleep(on_duration)



import logging
import socket
import sys
import time

from os.path import dirname

from . import rgb
from . import white
from . import rgbw

class Bridge:
    def send_command(self, part1, part2=0x00):
        message = bytearray([part1, part2, 0x55])

        # Send message multiple times to simulate a button press
        for i in range(0, 3):
            try:
                if self.protocol == 'udp':
                    self.sock.sendto(message, (self.address, self.port))
                elif self.protocol == 'tcp':
                    self.sock.send(message)

                    # Catch and discard the response
                    self.sock.recv(1024)
            except Exception as e:
                # Reconnect on failures
                self.create_connection()

                logging.error(e)

            # Keep the messages from flooding the device
            self.short_pause()

    def short_pause(self):
        # Hand-tuned value that gets decent performance of speed to miss ratios
        time.sleep(self.short_pause_duration)

    def long_pause(self):
        # Useful for nightlight features that require a long press
        time.sleep(self.long_pause_duration)

    def create_connection(self):
        if self.protocol == 'udp':
            self.sock = socket.socket(socket.AF_INET, # Internet
                                      socket.SOCK_DGRAM) # UDP
        elif self.protocol == 'tcp':
            self.sock = socket.socket(socket.AF_INET, # Internet
                                      socket.SOCK_STREAM) # TCP
            self.sock.connect((self.address, self.port))
        else:
            raise TypeError('Protocol "%s" is not a known protocol.' % self.protocol)

    def __init__(self, address='192.168.1.100', port=50000,
                 short_pause_duration=0.025,
                 long_pause_duration=0.1,
                 protocol='udp'):

        self.address = address
        self.port = port
        self.short_pause_duration = short_pause_duration
        self.long_pause_duration = long_pause_duration
        self.protocol = protocol

        self.create_connection()

        self.rgb = rgb.RGB(self)
        self.white = white.White(self)
        self.rgbw = rgbw.RGBW(self)



import time

def run(bridge, duration=15, delta=1, delay=0):
    '''Fade down the light at a specified delay.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    duration -- length of time the effect will last (default 15)
    delta -- amount of change between colors (default 1)
             Requirement: delta > 1
    delay -- delay between fade commands (default 0)
    '''

    # Keep track of time in effect
    start_time = time.time()

    # Ensure we do not get an infinite loop
    delta = int(delta)
    if delta == 0:
        raise RuntimeError('rainbow_fade:delta value cannot be 0')

    # Max selction of colors possible
    max_colors = 256

    # Main loop
    i = 0
    while True:
        bridge.set_color_hex(chr(i))

        # Check if effect duration has past
        if time.time() - start_time > duration:
            break

        time.sleep(delay)
        i = (delta + i) % max_colors



import random
import time

def run(bridge, duration=15, max_on_duration=0, max_off_duration=0.1, max_stutter=4,
        colors=['red', 'royal_blue', 'white']):
    '''An effect that simulates United States police flashers.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    duration -- length of time the effect will last (default 15)
    max_on_duration -- max length of time possible for the lights to remain in the on position (default 0)
    max_off_duration -- max length of time possible for the lights to remain in the off position (default 0.1)
    max_stutter -- max amount of times that an event will loop (default 4)
    colors -- list of possible colors to choose from (default: ['red', 'royal_blue', 'white']])
              `None`: Random selection of colors
    '''

    # Keep track of time in effect
    start_time = time.time()

    if colors:
        # Set white to be easily accessible via mode_down()
        if 'white' in colors:
            bridge.white()
            bridge.max_brightness()

        # Choose a color in colors to properly set max_brightness
        for color in colors:
            if color != 'white':
                bridge.set_color(color)
                bridge.max_brightness()
                break

    # Main loop
    while True:
        # Flash the lights off, then on
        if max_off_duration:
            # Possibly repeat the lights off effect
            this_stutter_count = random.random() * max_stutter
            i = 0
            while i < this_stutter_count:
                bridge.all_off()

                # Keep lights off for random length of time
                time.sleep(random.random() * max_off_duration)

                bridge.all_on()
                i += 1

        # Possibly repeat the color changing effect
        this_stutter_count = random.random() * max_stutter
        i = 0
        while i < this_stutter_count:
            if colors:
                # If colors are provided, choose a specified color
                current_color = colors[int(random.random() * len(colors))]
                if current_color == 'white':
                    bridge.white()
                else:
                    bridge.set_color(current_color)
            else:
                # If colors are NOT provided, choose a random color
                bridge.set_color_hex(chr(int(random.random() * 255)))

            # Keep lights off for random length of time
            time.sleep(random.random() * max_on_duration)
            i += 1

        # Check if effect duration has past
        if time.time() - start_time > duration:
            break




import time

def run(bridge, delay=0):
    '''Fade up light at a specified delay.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    delay -- delay between fade up commands (default 0)
    '''

    if hasattr(bridge, 'set_brightness') and callable(getattr(bridge, 'set_brightness')):
        for i in range(2,28):
            bridge.set_brightness(i)
            time.sleep(delay)
        return

    # Number of steps between min and max
    max_loop = 9

    # Main loop
    i = 0
    while i < max_loop:
        bridge.brightness_up()

        i += 1
        if i == max_loop:
            break

        time.sleep(delay)



import logging
import sys

class RGBW:
    zone_list = [0x42,
                 0x45,
                 0x47,
                 0x49,
                 0x4B,
                ]
    def effect(self, effect_name, args=[], effect_options={}):
        effect = sys.modules['wifileds.limitlessled.effects.%s' % effect_name]
        try:
            effect.run(self, *args, **effect_options)
        except AttributeError as e:
            logging.error('Effect "%s" failed due to missing lighting attribute: %s' % (effect_name, e))
            pass

    def __init__(self, parent):
        self.parent = parent
        self.long_pause = parent.long_pause
        self.short_pause = parent.short_pause

        self.color_map = {
            'violet': 0x00,
            'royal_blue': 0x10,
            'baby_blue': 0x20,
            'aqua': 0x30,
            'mint': 0x40,
            'seafoam_green': 0x50,
            'green': 0x60,
            'lime_green': 0x70,
            'yellow': 0x80,
            'yellow_orange': 0x90,
            'orange': 0xA0,
            'red': 0xB0,
            'pink': 0xC0,
            'fusia': 0xD0,
            'lilac': 0xE0,
            'lavendar': 0xF0
        }

    def zone_on(self, zone=0):
        # zone 0 = all on
        self.parent.send_command(self.zone_list[zone])

    def zone_off(self, zone=0):
        # zone 0 = all on
        if zone == 0:
            self.parent.send_command(0x41)
        else:
            self.parent.send_command(self.zone_list[zone]+1)

    def all_on(self):
        self.zone_on(0)

    def all_off(self):
        self.zone_off(0)

    def set_brightness(self, brightness, zone=0):
        # RGBW accepts values from 2-27
        if brightness < 2 or brightness > 27:
            raise KeyError('Brightness should be > 1 and < 28.')

        self.zone_on(zone)
        self.parent.long_pause()
        self.parent.send_command(0x4E, chr(int(brightness)))

    def max_brightness(self, zone=0):
        self.set_brightness(27, zone)

    def min_brightness(self, zone=0):
        self.set_brightness(2, zone)

    def mode_up(self):
        self.parent.send_command(0x4D)

    def speed_up(self):
        self.parent.send_command(0x44)

    def speed_down(self):
        self.parent.send_command(0x43)

    def max_speed(self):
        for i in range(0, 9):
            self.speed_up()

    def min_speed(self):
        for i in range(0, 9):
            self.speed_down()

    def set_color(self, color_name, zone=0):
        # If zone == 0, send to all. Otherwise send to specific zone
        try:
            self.set_color_hex(self.color_map[color_name], zone)
        except KeyError as e:
            raise KeyError('Color not found: %s' % e)

    def set_color_hex(self, color_value, zone=0):
        self.zone_on(zone)
        self.parent.long_pause()
        self.parent.send_command(0x40, color_value)

    def set_color_wheel(self, percentage, zone=0):
        if percentage < 0 or percentage > 1:
            raise ValueError('Wheel percentage should be > 0 and < 1.')
        self.set_color_hex(chr(int(float(255) * percentage)), zone)


    def white(self, zone=0):
        # If zone == 0, send to all. Otherwise send to specific zone
        self.parent.send_command(self.zone_list[zone])
        self.parent.long_pause()
        white_zones = [0xC2,
                       0xC5,
                       0xC7,
                       0xC9,
                       0xCB,
                      ]
        self.parent.send_command(white_zones[zone])

    def party_mode(self, number):
        description = {
            1: 'static white color',
            2: 'white color (gradual changes)',
            3: 'all colors (gradual changes)',
            4: 'red/green/blue (gradual changes)',
            5: '7 colors (jump changes)',
            6: '3 colors (jump changes)',
            7: 'red/green (jump changes)',
            8: 'red/blue (jump changes)',
            9: 'blue/green (jump changes)',
            10: 'white color (frequently blinks)',
            11: 'white color (glitters)',
            12: 'red color (frequently blinks)',
            13: 'red color (glitters)',
            14: 'green color (frequently blinks)',
            15: 'green color (glitters)',
            16: 'blue color (frequently blinks)',
            17: 'blue color (glitters)',
            18: 'yellow color (frequently blinks)',
            19: 'yellow color (glitters)',
            20: 'circulation mode',
        }
        print 'Attempting party mode: %s...' % description[number]

        self.white()
        for i in range(1, number):
            self.mode_up()

        print 'Party mode set.'



import logging
import sys

class White:
    def effect(self, effect_name, args=[], effect_options={}):
        effect = sys.modules['wifileds.limitlessled.effects.%s' % effect_name]
        try:
            effect.run(self, *args, **effect_options)
        except AttributeError as e:
            logging.error('Effect "%s" failed due to missing lighting attribute: %s' % (effect_name, e))
            pass

    def __init__(self, parent):
        self.parent = parent

    def all_on(self):
        self.parent.send_command(0x35)

    def all_off(self):
        self.parent.send_command(0x39)

    def brightness_up(self):
        self.parent.send_command(0x3C)

    def brightness_down(self):
        self.parent.send_command(0x34)

    def warmer(self):
        self.parent.send_command(0x3E)

    def cooler(self):
        self.parent.send_command(0x3F)

    def zone_on(self, zone):
        if zone == 1:
            self.parent.send_command(0x38)
        elif zone == 2:
            self.parent.send_command(0x3D)
        elif zone == 3:
            self.parent.send_command(0x37)
        elif zone == 4:
            self.parent.send_command(0x32)

    def zone_off(self, zone):
        if zone == 1:
            self.parent.send_command(0x3B)
        elif zone == 2:
            self.parent.send_command(0x33)
        elif zone == 3:
            self.parent.send_command(0x3A)
        elif zone == 4:
            self.parent.send_command(0x36)

    def full_all(self):
        self.all_on()
        self.parent.long_pause()
        self.parent.send_command(0xB5)

    def full_zone(self, zone):
        self.zone_on(zone)
        self.parent.long_pause()

        if zone == 1:
            self.parent.send_command(0xB8)
        elif zone == 2:
            self.parent.send_command(0xBD)
        elif zone == 3:
            self.parent.send_command(0xB7)
        elif zone == 4:
            self.parent.send_command(0xB2)

    def nightlight_all(self):
        self.all_off()
        self.parent.long_pause()
        self.parent.send_command(0xB9)

    def nightlight_zone(self, zone):
        self.zone_off(zone)
        self.parent.long_pause()

        if zone == 1:
            self.parent.send_command(0xBB)
        elif zone == 2:
            self.parent.send_command(0xB3)
        elif zone == 3:
            self.parent.send_command(0xBA)
        elif zone == 4:
            self.parent.send_command(0xB6)

    def max_brightness(self):
        for i in range(0, 10):
            self.brightness_up()

    def min_brightness(self):
        for i in range(0, 10):
            self.brightness_down()

    def max_cool(self):
        for i in range(0, 10):
            self.cooler()

    def max_warm(self):
        for i in range(0, 10):
            self.warmer()



import logging
import sys

class RGB:
    def effect(self, effect_name, args=[], effect_options={}):
        effect = sys.modules['wifileds.limitlessled.effects.%s' % effect_name]
        try:
            effect.run(self, *args, **effect_options)
        except AttributeError as e:
            logging.error('Effect "%s" failed due to missing lighting attribute: %s' % (effect_name, e))
            pass

    def __init__(self, parent):
        self.parent = parent
        self.long_pause = parent.long_pause
        self.short_pause = parent.short_pause

        self.color_map = {
            'violet': 0x00,
            'royal_blue': 0x10,
            'baby_blue': 0x20,
            'aqua': 0x30,
            'mint': 0x40,
            'seafoam_green': 0x50,
            'green': 0x60,
            'lime_green': 0x70,
            'yellow': 0x80,
            'yellow_orange': 0x90,
            'orange': 0xA0,
            'red': 0xB0,
            'pink': 0xC0,
            'fusia': 0xD0,
            'lilac': 0xE0,
            'lavendar': 0xF0
        }

    def all_on(self):
        self.parent.send_command(0x22)

    def all_off(self):
        self.parent.send_command(0x21)

    def brightness_up(self):
        self.parent.send_command(0x23)

    def brightness_down(self):
        self.parent.send_command(0x24)

    def mode_up(self):
        self.parent.send_command(0x27)

    def mode_down(self):
        self.parent.send_command(0x28)

    def speed_up(self):
        self.parent.send_command(0x25)

    def speed_down(self):
        self.parent.send_command(0x26)

    def set_color(self, color_name):
        try:
            self.set_color_hex(self.color_map[color_name])
        except KeyError as e:
            raise KeyError('Color not found: %s' % e)

    def set_color_hex(self, color_value):
        self.parent.send_command(0x20, color_value)

    def set_color_wheel(self, percentage):
        if percentage < 0 or percentage > 1:
            raise ValueError('Wheel percentage should be > 0 and < 1.')
        self.parent.send_command(0x20, chr(int(float(255) * percentage)))

    def max_brightness(self):
        for i in range(0, 9):
            self.brightness_up()

    def min_brightness(self):
        for i in range(0, 9):
            self.brightness_down()

    def max_speed(self):
        for i in range(0, 9):
            self.speed_up()

    def min_speed(self):
        for i in range(0, 9):
            self.speed_down()

    def white(self):
        for i in range(0, 20):
            self.mode_down()

    def party_mode(self, number):
        description = {
            1: 'static white color',
            2: 'white color (gradual changes)',
            3: 'all colors (gradual changes)',
            4: 'red/green/blue (gradual changes)',
            5: '7 colors (jump changes)',
            6: '3 colors (jump changes)',
            7: 'red/green (jump changes)',
            8: 'red/blue (jump changes)',
            9: 'blue/green (jump changes)',
            10: 'white color (frequently blinks)',
            11: 'white color (glitters)',
            12: 'red color (frequently blinks)',
            13: 'red color (glitters)',
            14: 'green color (frequently blinks)',
            15: 'green color (glitters)',
            16: 'blue color (frequently blinks)',
            17: 'blue color (glitters)',
            18: 'yellow color (frequently blinks)',
            19: 'yellow color (glitters)',
            20: 'circulation mode',
        }
        print 'Attempting party mode: %s...' % description[number]

        self.white()
        for i in range(1, number):
            self.mode_up()

        print 'Party mode set.'



import time

def run(bridge, duration=15, on_duration=0, off_duration=0):
    '''An effect that yellow factory warning flashers.

    Keyword arguments:
    bridge -- the bridge that will be controlled (required)
    duration -- length of time the effect will last (default 15)
    on_duration -- length of time for the lights to remain in the on position (default 0)
    off_duration -- length of time for the lights to remain in the off position (default 0)
    '''

    # Keep track of time in effect
    start_time = time.time()

    # Main loop
    while True:
        # Flash the lights off, then on
        bridge.all_off()
        time.sleep(off_duration)
        bridge.all_on()

        # Check if effect duration has past
        if time.time() - start_time > duration:
            break

        # Leave the bridge on white
        time.sleep(on_duration)



#!/usr/bin/env python
try:
      from setuptools import setup
except:
      from distutils.core import setup

setup(name='wifileds',
      version='1.0.6',
      description='Library for multiple WiFi LED vendors.',
      long_description=open('README.md').read(),
      author='Joaquin Casares',
      author_email='joaquin.casares AT gmail.com',
      url='http://github.com/joaquincasares/python-wifi-leds',
      packages=['wifileds', 
                'wifileds.limitlessled', 
                'wifileds.limitlessled.effects'],
      package_data={'': ['README.md']},
      keywords='wifi led leds l.e.d. light emitting diodes lights lamps bulbs lighting home automation network mesh future LimitlessLED EasyBulb AppLight AppLamp MiLight LEDme dekolight iLight Philips hue LIFX'
     )
